/*
 * Commodity related functions
 *
 * $Id: ix.c,v 1.5 2013/03/08 13:35:36 jacadcaps Exp $
 */

#include "globals.h"

/* public */
#include <clib/alib_protos.h>

/* private */
#include "ix.h"
#include "errorreq.h"
#include "rev.h"
#include "copyright.h"
#include "mui.h"

CxObj *custom;
CxObj *broker;
ULONG cxsig;

extern struct EmulLibEntry GATE_cxfunction;
#define CXFUNCTION &GATE_cxfunction

struct NewBroker newbroker = {
	NB_VERSION,
	APPNAME,
	APPNAME" "VERSIONSTRING,
	"Lightweight VNC client",
	NBU_UNIQUE | NBU_NOTIFY,
	COF_SHOW_HIDE,
	0,
	0,
	0
};


CxObj *bind_hotkey(STRPTR hotkey, int value)
{
	return HotKey(hotkey, newbroker.nb_Port, value);
}

int setup_newbroker(LONG priority)
{
	if (priority >= -128 && priority <= 127)
	{
		if ((newbroker.nb_Port = CreateMsgPort()))
		{
			cxsig = 1 << newbroker.nb_Port->mp_SigBit;

			newbroker.nb_Pri = priority;

			if ((broker = CxBroker(&newbroker, NULL)))
			{
				if ((custom = CxCustom(CXFUNCTION, 0)))
				{
					ActivateCxObj(broker, 1);
					return (TRUE);
				}
				else
				{
					errorreq(NULL, "Couldn't initialize newbroker", "Huho");
				}
			}
			else
			{
				errorreq(NULL, "Couldn't initialize newbroker", "Huho");
			}
		}
		else
		{
			errorreq(NULL, "Couldn't initialize newbroker", "Huho");
		}
	}
	else
	{
		errorreq(NULL, "Tooltype CX_PRIORITY out of range", "Bleh");
	}
	
	cleanup_newbroker();
	return (FALSE);
}


void cleanup_newbroker(void)
{
	CxMsg *msg;

	if (broker) DeleteCxObjAll(broker);	

	if (newbroker.nb_Port)
	{
		while ((msg = (CxMsg *)GetMsg(newbroker.nb_Port)))
		{
			ReplyMsg((struct Message *)msg);
		}
		DeleteMsgPort(newbroker.nb_Port);
	}
	
	broker = NULL;
	newbroker.nb_Port = NULL;
}


int process_ix(void)
{
	CxMsg *msg;
	LONG id, type;
	
	while ((msg = (CxMsg *)GetMsg(newbroker.nb_Port)))
	{
		//UBYTE *foo = (UBYTE *)msg; /* XXX: hack! */
		type = CxMsgType(msg);
		id = CxMsgID(msg);
		
		#if 0
		dprintf("evil: %ld, %ld, %ld, %ld, %ld, %ld, %ld, %ld\n", foo[0], foo[1], foo[2], foo[3], foo[4], foo[5], foo[6], foo[7]);
		dprintf("evil: %ld, %ld, %ld, %ld, %ld, %ld, %ld, %ld\n", foo[8], foo[9], foo[10], foo[11], foo[12], foo[13], foo[14], foo[15]);
		dprintf("evil: %ld, %ld, %ld, %ld, %ld, %ld, %ld, %ld\n", foo[16], foo[17], foo[18], foo[19], foo[20], foo[21], foo[22], foo[23]);
		dprintf("evil: %ld, %ld, %ld, %ld, %ld, %ld, %ld, %ld\n", foo[24], foo[25], foo[26], foo[27], foo[28], foo[29], foo[30], foo[31]);
		#endif

		ReplyMsg((struct Message *)msg);

		switch (type)
		{
	        case CXM_IEVENT:
	            switch (id)
				{
					case EVT_QUICKCONNECT:
						MUI_Key_Quick();
						break;
						
					case EVT_CAPTURETOGGLE:
						MUI_Key_Toggle();
						break;
						
					case EVT_DISCONNECT:
						MUI_Key_Disconnect();
						break;
				}
	            break;
			case CXM_COMMAND:
	            switch (id)
				{
					case CXCMD_DISABLE:
						ActivateCxObj(broker, 0);
						break;

					case CXCMD_ENABLE:
						ActivateCxObj(broker, 1);
						break;

					case CXCMD_APPEAR:
						MUI_Show();
						break;

					case CXCMD_DISAPPEAR:
						MUI_Hide();
						break;

					case CXCMD_KILL:
					case CXCMD_UNIQUE:
						return (PIX_QUIT);
						break;
				}
				break;
		}
	}

	return (PIX_CONTINUE);
}
